--  Blink --  Christmas tree blink package
--  Copyright (C) 2001, 2002 Free Software Foundation, Inc.
--  Written by Stephane Carrez (stcarrez@nerim.fr)
--
--  This file is free software; you can redistribute it and/or modify it
--  under the terms of the GNU General Public License as published by the
--  Free Software Foundation; either version 2, or (at your option) any
--  later version.
--
--  In addition to the permissions in the GNU General Public License, the
--  Free Software Foundation gives you unlimited permission to link the
--  compiled version of this file with other programs, and to distribute
--  those programs without any restriction coming from the use of this
--  file.  (The General Public License restrictions do apply in other
--  respects; for example, they cover modification of the file, and
--  distribution when not linked into another program.)
--
--  This file is distributed in the hope that it will be useful, but
--  WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
--  General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; see the file COPYING.  If not, write to
--  the Free Software Foundation, 59 Temple Place - Suite 330,
--  Boston, MA 02111-1307, USA.

pragma No_Run_Time;

with BSP.IO;
package Blink is

   subtype Light is BSP.Byte;

   L_1 : constant Light := 16#01#;
   L_2 : constant Light := 16#02#;
   L_3 : constant Light := 16#04#;
   L_4 : constant Light := 16#08#;
   L_5 : constant Light := 16#10#;
   L_6 : constant Light := 16#20#;
   L_7 : constant Light := 16#40#;
   L_8 : constant Light := 16#80#;

   type Repetition is new Natural range 0 .. 15;
   --  Repetition counter to repeat a blink sequence.

   type Time is new Natural range 0 .. 15;
   --  Time in 1/10 of seconds to wait before inverting the light.
   --  (1/2 of blink period)

   type Speed is record
      Wait   : Time;
      Repeat : Repetition;
   end record;
   --  Blink sequence definition.

   for Speed use
      record
         Wait   at 0 range 0 .. 3;
         Repeat at 0 range 4 .. 7;
      end record;
      for Speed'Size use 8;

   type Speed_Table is array (Natural range <>) of Speed;

   Local_Speeds : constant Speed_Table (1 .. 4) :=
     (1 => (Wait => 2, Repeat => 4),
      2 => (Wait => 5, Repeat => 2),
      3 => (Wait => 3, Repeat => 1),
      4 => (Wait => 8, Repeat => 3));
      --  5 => (Wait => 2, Repeat => 4),
      --  6 => (Wait => 1, Repeat => 6),
      --  7 => (Wait => 5, Repeat => 4));
   Speeds : Speed_Table (1 .. 255);
   pragma Import_Object (Speeds, "_start");

   type Sleep_Delay is new Natural range 0 .. 255;
   for Sleep_Delay'Size use 8;

   Base_Time   : constant Natural := 10;
   PWM_ON_Time : constant Sleep_Delay := Sleep_Delay (250 / Base_Time);
   PWM_OFF_Time: constant Sleep_Delay := Sleep_Delay (500 / Base_Time);

   procedure Sleep (N : Sleep_Delay);
   --  Wait for N * Base_Time micro-seconds.

   procedure Switch_Lights (Val : in Light; Duration : in Time);
   --  Switch ON or OFF the lights for the specified duration.
   --  Before switching the lights, we first synchronize with
   --  the mains by waiting for it to reach 0.
   --  The lights are switched ON using PWM pulses of 250us.

   procedure Wait_Zero_Crossing;
   --  Wait for the mains to cross 0.

   procedure Main_Loop;
   --  Run the blink sequence forever.

   Ios_Boot : BSP.IO.Port_Map;
   --  IOS mapping after the controller is reset (mapped at 0x1000).
   --  This is used during initialization to remap the IOS at 0x0000.

private
   pragma Import_Object (Ios_Boot, "_io_ports_startup");
   pragma No_Return (Main_Loop);
end Blink;
