/* ultoa.c -- Convert a long to a string
   Copyright (C) 2003 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@nerim.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

   ultoa() has been copied from FreeBSD vfprintf.c __ultoa() and adapted.

*/

/*-
 * Copyright (c) 1990, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Chris Torek.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdarg.h>
#include <limits.h>
#include <stddef.h>
#include <util.h>

/*
 * Macros for converting digits to letters and vice versa
 */
#define	to_char(n)	((n) + '0')

/*
 * Convert an unsigned long to ASCII for printf purposes, returning
 * a pointer to the first character of the string representation.
 * Octal numbers can be forced to have a leading zero; hex numbers
 * use the given digits.
 */
char*
ultoa (unsigned long val, char *endp, int base, int mode, ...)
{
  char *cp = endp;

  /*
   * Handle the three cases separately, in the hope of getting
   * better/faster code.
   */
  switch (base)
    {
    case 10:
      {
        char thousep;
        const char* grp;
        int ndig;

        if (val < 10)
          {	/* many numbers are 1 digit */
            *--cp = to_char(val);
            return (cp);
          }
        if (mode & ULTOA_GROUP)
          {
            va_list argp;

            va_start (argp, mode);
            thousep = va_arg (argp, int);
            grp = va_arg (argp, const char*);
            va_end (argp);
            ndig = 0;
          }

        do {
          *--cp = to_char(val % 10);

          /*
           * If (*grp == CHAR_MAX) then no more grouping
           * should be performed.
           */
          if (mode & ULTOA_GROUP)
            {
              ndig++;
              if (ndig == *grp && *grp != CHAR_MAX && val > 9)
                {
                  *--cp = thousep;
                  ndig = 0;
                  /*
                   * If (*(grp+1) == '\0') then we have to
                   * use *grp character (last grouping rule)
                   * for all next cases
                   */
                  if (*(grp+1) != '\0')
                    grp++;
                }
            }
          val /= 10;
        } while (val != 0);
      }
      break;

    case 8:
      do {
        *--cp = to_char(val & 7);
        val >>= 3;
      } while (val);
      if ((mode & ULTOA_OCT_ZERO) && *cp != '0')
        *--cp = '0';
      break;
        
    case 16:
    default:			/* oops */
      {
        const char* xdigs = (mode & ULTOA_UPPER
                             ? "0123456789ABCDEF"
                             : "0123456789abcdef");
        do {
          *--cp = xdigs[val & 15];
          val >>= 4;
        } while (val);
        break;
      }
    }
  return (cp);
}

