/* gdm-fillrect.c -- Fill Rectangle
   Copyright 2003 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@nerim.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <gdm/display.h>
#include <gdm/display-raw.h>

void
_gdm_fill_rectangle (gdm_display* dp, unsigned short x, unsigned short y,
                     unsigned short x1, unsigned short y1)
{
  gdm_mask emask;
  gdm_mask tmask;
  gdm_line* l = _gdm_get_raw_line (dp, y);
  gdm_mask smask = _gdm_start_mask[y & 7];
  unsigned short height;

  emask = _gdm_end_mask[y1 & 7];
  height = (y1 >> 3) - (y >> 3);
  if (height == 0)
    smask &= emask;

  /* Compute mask representing the touched region.  */
  tmask = _gdm_start_mask[x >> 4];
  tmask &= _gdm_end_mask[x1 >> 4];
  while (1)
    {
      unsigned char* p;
      unsigned short w;

      l->touched |= tmask;
      w = x1 - x + 1;
      p = &l->data[x];
      switch (dp->plot_mode)
        {
        case GDM_PLOT_OR:
        default:
          do {
            *p++ |= smask;
          } while (--w != 0);
          break;
      
        case GDM_PLOT_AND:
          do {
            *p++ &= ~smask;
          } while (--w != 0);
          break;

        case GDM_PLOT_XOR:
          do {
            *p++ ^= smask;
          } while (--w != 0);
          break;
        }
      if (height == 0) break;
      height--;
      if (height == 0)
        smask = emask;
      else
        smask = 0xff;
      l++;
    }
}

/*! Fill a rectangle with current gc mode.

    The rectangle defined by @b x, @b y, @b width, @b height is filled
    with the current graphical context.  The rectangle is clipped to the
    display and to the optional clipping region.

    @param dp  Display Manager
    @param x   X coordinate
    @param y   Y coordinate
    @param width Width of rectangle
    @param height Height of rectangle  */
void
gdm_fill_rectangle (gdm_display* dp, short x, short y,
                    unsigned short width, unsigned short height)
{
  short x1, y1;

  x1 = width;
  if (x1 <= 0)
    return;

  x1 += x - 1;
  if (x1 >= GDM_WIDTH)
    x1 = GDM_WIDTH - 1;
  if (x >= GDM_WIDTH)
    return;
  if (x < 0)
    x = 0;

  y1 = height;
  if (y1 <= 0)
    return;

  y1 += y - 1;
  if (y1 > GDM_HEIGHT)
    y1 = GDM_HEIGHT;
  if (y >= GDM_HEIGHT)
    return;
  if (y < 0)
    y = 0;

  _gdm_fill_rectangle (dp, x, y, x1, y1);
}
