/* gprint.c -- Generic Print function
   Copyright 2000, 2001, 2002 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@nerim.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <sys/param.h>
#include <sys/locks.h>
#include <sys/ports.h>
#include <sys/sio.h>
#include <sys/spi.h>
#include <lcd.h>
#include <time.h>
#include <sys/time.h>
#include <stddef.h>
#include <stdio.h>

typedef unsigned short (* var_to_string) (const void *var, char *buf,
                                          size_t len);
typedef int (* var_from_string) (void *var, char *buf);

struct var_type_def
{
  unsigned short size;
  var_to_string  to_string;
  var_from_string from_string;
};
typedef struct var_type_def var_typedef;

struct gprint_def
{
  const char  *title;
  const var_typedef *def;
};

struct var_ref
{
  var_typedef *type;
  void *ptr;
};

static size_t
ulong_to_string (const void *var, char *buf, size_t len)
{
  return snprintf (buf, len, "%ld", *((const unsigned long*) var));
}

const var_typedef type_ulong_dec = {
  sizeof (unsigned long),
  ulong_to_string,
  0
};

static size_t
timeval_to_string (const void *var, char *buf, size_t len)
{
  const struct timeval *tv = (struct timeval *) var;
  
  return snprintf (buf, len, "%ld sec", tv->tv_sec);
}

const var_typedef type_timeval_raw = {
  sizeof (struct timeval),
  timeval_to_string,
  0
};
const var_typedef type_timeval_duration = {
  sizeof (struct timeval),
  timeval_to_string,
  0
};
const var_typedef type_timeval_date = {
  sizeof (struct timeval),
  timeval_to_string,
  0
};

const struct gprint_def pdef[] = {
  { "Time",  &type_timeval_raw },
  { "Count", &type_ulong_dec },
};

const struct gprint_def light_stat_def[] = {
  { "Light ON count ", &type_ulong_dec },
  { "Light ON date  ", &type_timeval_date },
  { "Light ON time  ", &type_timeval_duration },
  { 0, 0 }
};

const struct gprint_def fan_stat_def[] = {
  { "Fan ON count ", &type_ulong_dec },
  { "Fan ON date  ", &type_timeval_date },
  { "Fan ON time  ", &type_timeval_duration },
  { 0, 0 }
};

int
format_variable (char *buf, size_t len, const void *var,
                 struct gprint_def *pdef, int member)
{
  size_t size;
  
  while (member > 0)
    {
      if (pdef->title == 0)
        return 0;

      var = ((const char *) (var) + pdef->def->size);
      member--;
      pdef++;
    }
  size = snprintf (buf, len, "%s", pdef->title);
  buf += size;
  len -= size;
  size += pdef->def->to_string (var, buf, len);
  return size;
}
