/* gdm-benchs.c -- Benchmark for Graphic Display Manager Operations
   Copyright (C) 2003 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@nerim.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <benchs.h>
#include <gdm/display.h>

gdm_display display;

static void
bench_fillrect (bench_t* b, const char* mode)
{
  bench_start (b);
  gdm_fill_rectangle (&display, 3, 3, 10, 10);
  bench_stop (b);
  bench_report (b, "gdm_fill_rectangle (%s) [3, 3, 10, 10]", mode);

  bench_start (b);
  gdm_fill_rectangle (&display, 0, 0, 128, 64);
  bench_stop (b);
  bench_report (b, "gdm_fill_rectangle (%s) [0, 0, 128, 64]", mode);
}

static void
bench_plot (bench_t* b, const char* mode)
{
  bench_start (b);
  gdm_draw_point (&display, 10, 10);
  bench_stop (b);
  bench_report (b, "gdm_draw_point (%s)", mode);
}

static void
bench_line (bench_t* b, const char* mode)
{
  bench_start (b);
  gdm_draw_line (&display, 0, 0, 128, 0);
  bench_stop (b);
  bench_report (b, "gdm_draw_line (%s) 0,0 -> 128,0", mode);

  bench_start (b);
  gdm_draw_line (&display, 0, 0, 0, 64);
  bench_stop (b);
  bench_report (b, "gdm_draw_line (%s) 0,0 -> 0,64", mode);

  bench_start (b);
  gdm_draw_line (&display, 0, 0, 64, 64);
  bench_stop (b);
  bench_report (b, "gdm_draw_line (%s) 0,0 -> 64,64", mode);

  bench_start (b);
  gdm_draw_line (&display, 0, 0, 128, 64);
  bench_stop (b);
  bench_report (b, "gdm_draw_line (%s) 0,0 -> 128,64", mode);
}

static void
bench_refresh (bench_t* b)
{
  gdm_refresh (&display);

  bench_start (b);
  gdm_refresh (&display);
  bench_stop (b);
  bench_report (b, "gdm_display_refresh (no need)");

  bench_start (b);
  gdm_touch (&display, 0, 0, 10, 10);
  bench_stop (b);
  bench_report (b, "gdm_touch [0, 0, 10, 10]");

  bench_start (b);
  gdm_refresh (&display);
  bench_stop (b);
  bench_report (b, "gdm_display_refresh [0, 0, 10, 10]");

  bench_start (b);
  gdm_touch (&display, 0, 0, 128, 64);
  bench_stop (b);
  bench_report (b, "gdm_touch [0, 0, 128, 64]");

  bench_start (b);
  gdm_refresh (&display);
  bench_stop (b);
  bench_report (b, "gdm_display_refresh [0, 0, 128, 64]");
}

int
main ()
{
  bench_t b;

  gdm_initialize (&display);
  bench_init (&b);

  gdm_set_gc (&display, GDM_PLOT_XOR);
  bench_fillrect (&b, "XOR");
  bench_plot (&b, "XOR");
  bench_line (&b, "XOR");

  gdm_set_gc (&display, GDM_PLOT_AND);
  bench_fillrect (&b, "AND");
  bench_plot (&b, "AND");
  bench_line (&b, "AND");

  gdm_set_gc (&display, GDM_PLOT_OR);
  bench_fillrect (&b, "OR");
  bench_plot (&b, "OR");
  bench_line (&b, "OR");

  bench_refresh (&b);
  return 0;
}
