/* ltime-test.c -- Localtime test
   Copyright 2001 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of EBCS.

EBCS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

EBCS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with EBCS; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/*! @page lcd-test Test program for the LCD operations.

  This program tests the LCD operation by displaying patterns on
  the LCD panel.

  @htmlonly
  Source file: <a href="lcd-test_c-source.html">lcd-test.c</a>
  @endhtmlonly

 */

#include <sys/param.h>
#include <gel/timer.h>
#include <sys/sio.h>
#include <sys/interrupts.h>

#include <test.h>
#include <stdio.h>
#include <benchs.h>
#include <time.h>

#ifdef USE_INTERRUPT_TABLE

/* Interrupt table used to connect our GEL interrupt handlers.

   Note: the `XXX_handler: foo' notation is a GNU extension which is
   used here to ensure correct association of the handler in the struct.
   This is why the order of handlers declared below does not follow
   the HC11 order.  */
struct interrupt_vectors __attribute__((section(".vectors"))) vectors = 
{
  res0_handler:           fatal_interrupt, /* res0 */
  res1_handler:           fatal_interrupt,
  res2_handler:           fatal_interrupt,
  res3_handler:           fatal_interrupt,
  res4_handler:           fatal_interrupt,
  res5_handler:           fatal_interrupt,
  res6_handler:           fatal_interrupt,
  res7_handler:           fatal_interrupt,
  res8_handler:           fatal_interrupt,
  res9_handler:           fatal_interrupt,
  res10_handler:          fatal_interrupt, /* res 10 */
  sci_handler:            fatal_interrupt, /* sci */
  spi_handler:            fatal_interrupt, /* spi */
  acc_overflow_handler:   fatal_interrupt, /* acc overflow */
  acc_input_handler:      fatal_interrupt,
  output5_handler:        fatal_interrupt, /* out compare 5 */
  output4_handler:        fatal_interrupt, /* out compare 4 */
  output3_handler:        fatal_interrupt, /* out compare 3 */
  output2_handler:        fatal_interrupt, /* out compare 2 */
  output1_handler:        fatal_interrupt, /* out compare 1 */
  capture3_handler:       fatal_interrupt, /* in capt 3 */
  capture2_handler:       fatal_interrupt, /* in capt 2 */
  capture1_handler:       fatal_interrupt, /* in capt 1 */
  irq_handler:            fatal_interrupt, /* IRQ */
  xirq_handler:           fatal_interrupt, /* XIRQ */
  swi_handler:            fatal_interrupt, /* swi */
  illegal_handler:        fatal_interrupt, /* illegal */
  cop_fail_handler:       fatal_interrupt,
  cop_clock_handler:      fatal_interrupt,

  /* What we really need.  */
  timer_overflow_handler: timer_overflow_interrupt,
  rtii_handler:           timer_interrupt,
  reset_handler:          _start
};

#endif

void localtime_benchs (void);

void
localtime_benchs ()
{
  bench_t b;
  struct tm t;
  time_t n;
  
  bench_init (&b);

  n = 0xffffffff;
  bench_start (&b);
  localtime_r (&n, &t);
  bench_stop (&b);
  bench_report (&b, "localtime_r (%ld-%ld-%ld)",
		(long) (t.tm_year + 1900),
		(long) (t.tm_mon),
		(long) (t.tm_mday));

  n = 0x80000000;
  bench_start (&b);
  localtime_r (&n, &t);
  bench_stop (&b);
  bench_report (&b, "localtime_r (%ld-%ld-%ld)",
		(long) (t.tm_year + 1900),
		(long) (t.tm_mon),
		(long) (t.tm_mday));

  n = 0x40000000;
  bench_start (&b);
  localtime_r (&n, &t);
  bench_stop (&b);
  bench_report (&b, "localtime_r (%ld-%ld-%ld)",
		(long) (t.tm_year + 1900),
		(long) (t.tm_mon),
		(long) (t.tm_mday));

  n = 0x20000000;
  bench_start (&b);
  localtime_r (&n, &t);
  bench_stop (&b);
  bench_report (&b, "localtime_r (%ld-%ld-%ld)",
		(long) (t.tm_year + 1900),
		(long) (t.tm_mon),
		(long) (t.tm_mday));
}

int
main ()
{
  int result;
  unsigned long i;

  serial_init ();

  timer_initialize ();
  test_repeat = 100;
  test_depend ("serial:benchs");
  test_start ("localtime-test");
  for (i = 0; i < test_repeat; i++)
    {
    }
  result = test_end ("localtime-test");
  if (result == 0)
    {
      localtime_benchs ();
    }
  return result;
}
